-- SaberUniquesScript
-- Author: Vicevirtuoso
-- DateCreated: 3/22/2014 5:22:32 PM
--------------------------------------------------------------
include("FLuaVector.lua")
-------------------------------------------------------------------------------------------------------------------------
-- JFD_IsCivilisationActive
-------------------------------------------------------------------------------------------------------------------------
local civilizationID = GameInfoTypes.CIVILIZATION_BRITAIN_FSN
function JFD_IsCivilisationActive(civilisationID)
	for iSlot = 0, GameDefines.MAX_MAJOR_CIVS-1, 1 do
		local slotStatus = PreGame.GetSlotStatus(iSlot)
		if (slotStatus == SlotStatus["SS_TAKEN"] or slotStatus == SlotStatus["SS_COMPUTER"]) then
			if PreGame.GetCivilization(iSlot) == civilisationID then
				return true
			end
		end
	end

	return false
end

if JFD_IsCivilisationActive(civilizationID) then
print("Arturia Pendragon has been summoned as Saber servant to fight in the Sid Meier's Holy Grail War.")
end

-- IsUsingCBP
function IsUsingCBP()
	local CBP = "d1b6328c-ff44-4b0d-aad7-c657f83610cd"
	for _, mod in pairs(Modding.GetActivatedMods()) do
		if (mod.ID == CBP) then
			return true
		end
	end
	return false
end
local isUsingCBP = IsUsingCBP()
--=============================

--Cache the type of the UU, UB, dummy UB, and the tech requirements/strength upgrades for KOTRs. Makes things faster.
local iCourtType = GameInfoTypes.BUILDING_ARTHURIAN_COURT
--local iSpecialistType = GameInfoTypes.SPECIALIST_GREAT_GENERAL_SABERMOD
local iKOTRType = GameInfoTypes.UNIT_KOTR_FSN

local iLWarfare = GameInfoTypes.POLICY_LIGHTNING_WARFARE
local iLWarfarePromo = GameInfoTypes.PROMOTION_FAST_GENERAL

local iMaxCivs = GameDefines.MAX_MAJOR_CIVS

local combstr1 = GameInfo.Units["UNIT_KNIGHT"].Combat + 2
local combstr2 = GameInfo.Units["UNIT_LANCER"].Combat + 3
local combstr4 = GameInfo.Units["UNIT_WWI_TANK"].Combat + 6
local combstr5 = GameInfo.Units["UNIT_TANK"].Combat + 7

function KotRT_CSFix()
	if isUsingCBP then
		return 40
	else
		return GameInfo.Units["UNIT_CAVALRY"].Combat + 4
	end
end
local combstr3 = KotRT_CSFix()

local tKOTRTable = {
	[GameInfoTypes.TECH_CHIVALRY] = combstr1,
	[GameInfoTypes.TECH_METALLURGY] = combstr2, 
	[GameInfoTypes.TECH_MILITARY_SCIENCE] = combstr3, -- In VP Cavalry is ranged and weaker than Lancer
	[GameInfoTypes.TECH_COMBUSTION] = combstr4, 
	[GameInfoTypes.TECH_COMBINED_ARMS] = combstr5, 
	[GameInfoTypes.TECH_LASERS] = 110
	}

--When a Knight of the Round Table is killed, put its owner in anarchy for a number of turns based on game speed (2 at Standard).
function OnKOTRKillAnarchy(iKiller, iKilled, iUnitType)
	if iUnitType == iKOTRType and iKilled ~= 63 then
		local pKilled = Players[iKilled]
		if not pKilled:IsMinorCiv() then
			local iResearchLevel = GameInfo.GameSpeeds[PreGame.GetGameSpeed()].ResearchPercent / 100
			local iAnarchyTurns = math.floor(iResearchLevel * 3)
			pKilled:ChangeAnarchyNumTurns(iAnarchyTurns)
			if iKilled == Game:GetActivePlayer() then
				Events.GameplayAlertMessage(Locale.ConvertTextKey("TXT_KEY_ALERT_SABER_KOTR_DIED", iAnarchyTurns))
			end
		end
	end
end
if JFD_IsCivilisationActive(civilizationID) then
GameEvents.UnitKilledInCombat.Add(OnKOTRKillAnarchy)
end

--Increase KOTR strength function. Since doing the Madoka-style function will burn through GP names and the KOTR won't be as ubiquitous
--as Magical Girls, we will just be using SetBaseCombatStrength() for now.
function OnMoveKOTRStrengthUpkeep(iPlayer, iUnitID, iX, iY)
	local pPlayer = Players[iPlayer]
	if (pPlayer:GetCivilizationType() == civilizationID) and iX > 0 and iY > 0 then
		local pUnit = pPlayer:GetUnitByID(iUnitID)
		if pUnit then
			if pUnit:GetUnitType() == iKOTRType and pUnit:GetPlot():GetOwner() == iPlayer then
				local pTeam = Teams[pPlayer:GetTeam()]
				for iTech, iStrength in pairs(tKOTRTable) do
					if pTeam:IsHasTech(iTech) and iStrength > pUnit:GetBaseCombatStrength() then
						pUnit:SetBaseCombatStrength(iStrength)
					end
				end
			end
		end
	end
end

if JFD_IsCivilisationActive(civilizationID) then
GameEvents.UnitSetXY.Add(OnMoveKOTRStrengthUpkeep)
end

function OnTurnKOTRStrengthUpkeep(iPlayer)
	local pPlayer = Players[iPlayer]
	if (pPlayer:GetCivilizationType() == civilizationID) then
		for pUnit in pPlayer:Units() do
			if pUnit:GetUnitType() == iKOTRType then
				local pTeam = Teams[pPlayer:GetTeam()]
				for iTech, iStrength in pairs(tKOTRTable) do
					if pTeam:IsHasTech(iTech) and iStrength > pUnit:GetBaseCombatStrength() then
						pUnit:SetBaseCombatStrength(iStrength)
					end
				end
			end
		end
	end
end
if JFD_IsCivilisationActive(civilizationID) then
GameEvents.PlayerDoTurn.Add(OnTurnKOTRStrengthUpkeep)
end

--For every two Arthurian Courts in an Empire, put a copy of the dummy building which grants Great General progress in the Capital.

function GeneralProgressInCapitalFromCourts(iPlayer)
	local pPlayer = Players[iPlayer]
	if (pPlayer:GetCivilizationType() == civilizationID) then
		local pCapital = pPlayer:GetCapitalCity()
		if pCapital then
			local iTotalBuildings = 0;
			for pCity in pPlayer:Cities() do
				if pCity:IsHasBuilding(iCourtType) then
					iTotalBuildings = iTotalBuildings + 1
				end
			end
		iTotalBuildings = math.floor(iTotalBuildings / 2)
		--pCapital:ChangeSpecialistGreatPersonProgressTimes100(iSpecialistType, iTotalBuildings)
		pPlayer:ChangeCombatExperience(iTotalBuildings)
		end
	end
end

if JFD_IsCivilisationActive(civilizationID) and not isUsingCBP then
	GameEvents.PlayerDoTurn.Add(GeneralProgressInCapitalFromCourts)
end


function OnAdoptLightningWarfare(iPlayer, iPolicyID)
	if iPolicyID == iLWarfare then
		local pPlayer = Players[iPlayer]
		for unit in pPlayer:Units() do
			if unit:GetUnitType() == iKOTRType then
				unit:SetHasPromotion(iLWarfarePromo, true)
			end
		end
	end
end

if JFD_IsCivilisationActive(civilizationID) then
GameEvents.PlayerAdoptPolicy.Add(OnAdoptLightningWarfare)
end

function OnMoveLightningWarfare(iPlayer, iUnitID, iX, iY)
	local pPlayer = Players[iPlayer]
	if (pPlayer:GetCivilizationType() == civilizationID) and iX > 0 and iY > 0 then
		if pPlayer:HasPolicy(iLWarfare) then
			local pUnit = pPlayer:GetUnitByID(iUnitID)
			if pUnit then
				if pUnit:GetUnitType() == iKOTRType and not pUnit:IsHasPromotion(iLWarfarePromo) then
					pUnit:SetHasPromotion(iLWarfarePromo, true)
					pUnit:SetMoves(pUnit:MaxMoves())
				end
			end
		end
	end
end

if JFD_IsCivilisationActive(civilizationID) then
GameEvents.UnitSetXY.Add(OnMoveLightningWarfare)
end

--=========================================================================
-- Saber new trait
-- Food is added to Production when training Units
function SaberTraitFoodtoUnitProduction(iPlayer)
	local pPlayer = Players[iPlayer]
	if (pPlayer:GetCivilizationType() == civilizationID) then 
		for pCity in pPlayer:Cities() do
			local FoodYield = pCity:GetYieldRate(YieldTypes.YIELD_FOOD)
			local iFoodSurplus = FoodYield  - pCity:FoodConsumption(true, 0)
			if pCity:IsProductionUnit() then
				local unitProduction = pCity:GetProductionUnit()
				pCity:ChangeUnitProduction(unitProduction, iFoodSurplus)
				Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(pCity:Plot():GetX(), pCity:Plot():GetY()))), "[COLOR_POPUP_TEXT][ICON_FOOD] Food added to [ICON_PRODUCTION] Unit Production[ENDCOLOR]", 3)
				print("F/SN Saber is being fed by Shirou. Nom nom nom.")
			end
		end
	end		
end			
	
if JFD_IsCivilisationActive(civilizationID) then
GameEvents.PlayerDoTurn.Add(SaberTraitFoodtoUnitProduction)
end	


-- Saber Lily UA promotion - Excess happiness grants strength to all military units
local PromotionTraitSaberLily = GameInfoTypes.PROMOTION_TRAIT_SABERLILY
function SaberTraitHappinesstoStrength(iPlayer)
	local pPlayer = Players[iPlayer]
	if (pPlayer:GetCivilizationType() == civilizationID) then 
		local ExHappiness = pPlayer:GetExcessHappiness()
		--local NumCities = pPlayer:GetNumCities()
		--local CombatBonus = math.ceil(ExHappiness/NumCities)
		for pUnit in pPlayer:Units() do
			local iStrength = GameInfo.Units[pUnit:GetUnitType()].Combat
			if pUnit:IsHasPromotion(PromotionTraitSaberLily) then
				if ExHappiness > 10 and ExHappiness <= 40 then
					pUnit:SetBaseCombatStrength(iStrength +1)				
				elseif ExHappiness > 40 and ExHappiness <= 70 then
					pUnit:SetBaseCombatStrength(iStrength +2)
				elseif ExHappiness > 70 then --and ExHappiness <= 100 then
					pUnit:SetBaseCombatStrength(iStrength +3)
				--elseif ExHappiness > 100 then
				--	pUnit:SetBaseCombatStrength(iStrength +4)
				else
				end			
			end
		end
	end		
end			
	
if JFD_IsCivilisationActive(civilizationID) then
GameEvents.PlayerDoTurn.Add(SaberTraitHappinesstoStrength)
end

-- Multipurpose dummy policy	
function SaberTraitDummyPolicy()
    for iPlayerLoop = 0, GameDefines.MAX_MAJOR_CIVS-1, 1 do
        local pPlayer = Players[iPlayerLoop]
        if (pPlayer:GetCivilizationType() == civilizationID) then
            pPlayer:SetNumFreePolicies(1)
            pPlayer:SetNumFreePolicies(0)
            pPlayer:SetHasPolicy(GameInfoTypes.POLICY_SABER_DUMMY, true)
        end
    end
end

if (Modding.OpenSaveData().GetValue("Seibaa") == nil) then
    SaberTraitDummyPolicy()
    Modding.OpenSaveData().SetValue("Seibaa", 1)
end

-- +1 Golden Age Point to the city where a Knight of the Round Table is born (VoxPopuli only)
if JFD_IsCivilisationActive(civilizationID) then
LuaEvents.SerialEventUnitCreatedGood.Add(
function(iPlayer, iUnit, _, iUnitType, _, iCiv)
--	print("Unit created for Player "..iPlayer.." Unit "..iUnit.." UnitType "..iUnitType.." Civ "..iCiv..".")
	local pPlayer = Players[iPlayer]
	local pUnit = pPlayer:GetUnitByID(iUnit)
	local unitInfo = GameInfo.Units[pUnit:GetUnitType()]
	local pPlot = pUnit:GetPlot()
	if (iCiv == civilizationID) and pUnit:GetUnitType() == iKOTRType and isUsingCBP then
		if (pPlot:IsCity()) then
			pCity = pPlot:GetPlotCity()
			if (pCity:GetOwner() == iPlayer) then
				Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), GameInfoTypes.YIELD_GOLDEN_AGE_POINTS, 1)
				Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(pPlot:GetX(), pPlot:GetY()))), "[COLOR_POPUP_TEXT]A new valorous Knight is born![ENDCOLOR]", 2);
				Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(pPlot:GetX(), pPlot:GetY()))), "[COLOR_POPUP_TEXT]+1[ICON_GOLDEN_AGE] to the City[ENDCOLOR]", 3);
			end
		end
	end
end)
end


-- CBP UA Promotion: When killing an enemy unit, all adjacent enemy units get 5 Damage
function Ekusukaribaaa(iAttackingPlayer, iAttackingUnit, attackerDamage, attackerFinalDamage, attackerMaxHP, iDefendingPlayer, iDefendingUnit, defenderDamage, defenderFinalDamage, defenderMaxHP, iInterceptingPlayer, iInterceptingUnit, interceptorDamage, plotX, plotY)
	local pAttPlayer = Players[iAttackingPlayer]
	if pAttPlayer then -- In case the combat was aborted...
		local pDefPlayer = Players[iDefendingPlayer]
		local pDefendedPlot = Map.GetPlot(plotX, plotY)
		local pDefUnit = pDefPlayer:GetUnitByID(iDefendingUnit)
		local pAttUnit = pAttPlayer:GetUnitByID(iAttackingUnit)
		local defenderHealth = defenderMaxHP - defenderFinalDamage
		local pPlot = pDefUnit:GetPlot()
		if defenderHealth <= 0 and pAttUnit:IsHasPromotion(GameInfoTypes["PROMOTION_EXCALIBUR"]) then
			for direction = 0, DirectionTypes.NUM_DIRECTION_TYPES - 1, 1 do
				local pAdjacentPlot = Map.PlotDirection(pPlot:GetX(), pPlot:GetY(), direction)
				if pAdjacentPlot and pAdjacentPlot:IsUnit() then
					local AdjUnit= pAdjacentPlot:GetUnit()
					local pPlayerAdjUnit = Players[AdjUnit:GetOwner()]
					local pPlayerAdjUnitTeam = pPlayerAdjUnit:GetTeam()
					local pAttPlayerTeam = pAttPlayer:GetTeam()
					if pPlayerAdjUnit ~= pAttPlayer then 
						if pPlayerAdjUnit:IsBarbarian() or pAttPlayerTeam:IsAtWar(pPlayerAdjUnitTeam)  then
							AdjUnit:ChangeDamage(5, iAttackingPlayer)
							Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(pPlot:GetX(), pPlot:GetY()))), "[ICON_IDEOLOGY_AUTOCRACY][COLOR_YELLOW]EKUSUKARIBA~![ENDCOLOR]", 2)
						end
					end
				end
			end
		end
	end
end

if JFD_IsCivilisationActive(civilizationID) then
GameEvents.CombatResult.Add(Ekusukaribaaa)
end

print("F/SN Saber functions loaded")